#include <cjson/cJSON.h>
#include "wifichange.h"
#include "mqtt.h"
#include "common.h"
#include "judg.h"

char current_ssid[SSID_MAX_LEN] = {0};//用于存现在已连接WiFi

int find_ssid(wifi_info_t list[], int count, const char *ssid) {//找重复的WiFi名称
    for (int i = 0; i < count; i++) {
        if (strcmp(list[i].ssid, ssid) == 0) {
            return i;
        }
    }
    return -1;
}

int cmp_signal_desc(const void *a, const void *b) {//进行qsort前，此未降序操作
    wifi_info_t *wa = (wifi_info_t *)a;
    wifi_info_t *wb = (wifi_info_t *)b;
    return wb->signal - wa->signal;
}

void get_current_wifi() {
    //显示已连接代码
    FILE *fp = popen("nmcli -t -f ACTIVE,SSID dev wifi", "r");
    if (!fp) {
        my_zlog_error("{\"error\": \"无法执行扫描命令\"}");
        return;
    }

    char line[256];
    while (fgets(line, sizeof(line), fp)) {
        char *active = strtok(line, ":");
        char *ssid = strtok(NULL, "\n");
        if (active && ssid && strcmp(active, "yes") == 0 && strlen(ssid) > 0) {
            strncpy(current_ssid, ssid, SSID_MAX_LEN - 1);
            current_ssid[SSID_MAX_LEN - 1] = '\0';
            break;
        }
    }
    pclose(fp);
}

void scan_wifi_json() { //显示以连接WiFi名称和周围已有的WiFi和过滤信号强度低于50的WiFi
    
    get_current_wifi();
    char line[100];
    //显示WiFi名称与信号强度
    FILE *fp = popen("nmcli -t -f SSID,SIGNAL dev wifi list", "r");
    if (!fp) {
        my_zlog_error("{\"error\": \"无法执行扫描命令\"}");
        return;
    }

    wifi_info_t wifi_list[MAX_WIFI_LIST];
    int wifi_count = 0;

    while (fgets(line, sizeof(line), fp)) {
        char *ssid = strtok(line, ":");
        char *signal_str = strtok(NULL, "\n");
        if (!ssid || !signal_str) continue;
        if (strlen(ssid) == 0) continue;

        int signal = atoi(signal_str);
        int idx = find_ssid(wifi_list, wifi_count, ssid);//去重 + 只保留信号最强的
        if (idx == -1) {
            if (wifi_count < MAX_WIFI_LIST) {
                strncpy(wifi_list[wifi_count].ssid, ssid, SSID_MAX_LEN - 1);
                wifi_list[wifi_count].ssid[SSID_MAX_LEN - 1] = '\0';
                wifi_list[wifi_count].signal = signal;
                wifi_count++;
            }
        } else {
            if (signal > wifi_list[idx].signal) {
                wifi_list[idx].signal = signal;
            }
        }
    }
    pclose(fp);

    qsort(wifi_list, wifi_count, sizeof(wifi_info_t), cmp_signal_desc);//通过信号强度进行排序

    //将WiFi信息转化为json格式
    cJSON *root = cJSON_CreateObject();
    cJSON *body = cJSON_CreateObject();
    cJSON *head = cJSON_CreateObject();
    cJSON_AddStringToObject(body, "current_connected", current_ssid[0] ? current_ssid : "");

    cJSON *array = cJSON_CreateArray();
    for (int i = 0; i < wifi_count; i++) {
        if (wifi_list[i].signal < 50) continue;  // 过滤信号小于50的
        cJSON *item = cJSON_CreateObject();
        cJSON_AddStringToObject(item, "ssid", wifi_list[i].ssid);
        cJSON_AddNumberToObject(item, "strength", wifi_list[i].signal);
        cJSON_AddItemToArray(array, item);
    }
    cJSON_AddItemToObject(body, "available_wifi", array);
    cJSON_AddNumberToObject(head, "message_type", 3003);
    cJSON_AddItemToObject(root, "body", body);
    cJSON_AddItemToObject(root, "head",head);

    char *json_str = cJSON_Print(root);
    if (json_str) {
        my_zlog_debug("%s", json_str);
        topic_middle_value();
        mosquitto_publish(mosq, NULL, TOPIC3, strlen(json_str), json_str, 0, false); 
        free(json_str);
    }
    cJSON_Delete(root);
}

int connect_wifi(const char* ssid, const char* password) {//连接WiFi
    char cmd[CMD_BUFFER_SIZE];
    snprintf(cmd, sizeof(cmd), "nmcli dev wifi connect '%s' password '%s' 2>&1", ssid, password);

    my_zlog_debug("🔌 正在连接 Wi-Fi：%s...", ssid);

    FILE *fp = popen(cmd, "r");
    if (!fp) {
        my_zlog_warn("❌ 无法执行连接命令。");
        return -1;
    }

    char output[512] = {0};
    fread(output, 1, sizeof(output) - 1, fp);
    pclose(fp);

    if (strstr(output, "successfully activated")) {
        my_zlog_debug("✅ 已成功连接到：%s", ssid);
        return 0;
    } else if (strstr(output, "No network with SSID")) {
        my_zlog_warn("❌ 连接失败：Wi-Fi '%s' 不存在。", ssid);
        return -2;
    } else if (strstr(output, "Secrets were required") || strstr(output, "incorrect password")) {
        my_zlog_warn("❌ 连接失败：密码错误。");
        return -3;
    } else {
        my_zlog_error("❌ 连接失败，未知错误：%s", output);
        return -4;
    }
}

int connect_wifi_with_fallback(const char* ssid, const char* password, const char* fallback_ssid) {//如果连接失败则回退
    int ret = connect_wifi(ssid, password);
    if (ret != 0 && fallback_ssid && fallback_ssid[0] != 0) {
        my_zlog_warn("⚠️ 连接失败，尝试回退连接原Wi-Fi：%s", fallback_ssid);
        char cmd[CMD_BUFFER_SIZE];
        snprintf(cmd, sizeof(cmd), "nmcli dev wifi connect '%s' 2>&1", fallback_ssid);
        FILE *fp = popen(cmd, "r");
        if (fp) {
            char output[512] = {0};
            fread(output, 1, sizeof(output) - 1, fp);
            pclose(fp);
            if (strstr(output, "successfully activated")) {
                my_zlog_debug("✅ 回退连接成功：%s", fallback_ssid);
            } else {
                my_zlog_warn("❌ 回退连接失败：%s", output);
            }
        } else {
            my_zlog_error("❌ 无法执行回退连接命令。");
        }
    }
    return ret;
}

int change_wifi_connect(const char *wifi_ssid, const char *wifi_password) {//改变WiFi连接
    get_current_wifi(current_ssid, sizeof(current_ssid));

    int ret = connect_wifi_with_fallback(wifi_ssid, wifi_password, current_ssid);
    my_zlog_notice("返回代码：%d", ret);
    return ret;
}

int can_access_internet() {//测试网络是否连接成功
    int ret = system("ping -c 1 -W 1 baidu.com > /dev/null 2>&1");
    return ret == 0;  // 返回 0 表示 ping 成功
}

int delete_wifi_by_ssid(const char* ssid) {//删除对应wifi
    if (ssid == NULL || strlen(ssid) == 0) {
        fprintf(stderr, "SSID is empty.");
        return -1;
    }

    char command[256];

    // 构造 nmcli 删除连接命令
    snprintf(command, sizeof(command), "nmcli -t -f NAME,TYPE connection show | grep '^%s:wifi$' | cut -d: -f1", ssid);

    // 打开管道读取匹配到的连接名
    FILE* fp = popen(command, "r");
    if (!fp) {
        perror("popen failed");
        return -1;
    }

    char conn_name[128];
    if (fgets(conn_name, sizeof(conn_name), fp) == NULL) {
        my_zlog_debug("No matching WiFi connection found for SSID: %s", ssid);
        pclose(fp);
        return -1;
    }
    pclose(fp);

    // 去除末尾换行符
    conn_name[strcspn(conn_name, "\n")] = 0;

    // 构造删除命令
    snprintf(command, sizeof(command), "nmcli connection delete \"%s\"", conn_name);
    int ret = system(command);
    if (ret == 0) {
        my_zlog_debug("Successfully deleted WiFi connection: %s", conn_name);
    } else {
        my_zlog_warn("Failed to delete WiFi connection: %s", conn_name);
    }

    return ret;
}


// 检查文件是否存在，不存在则创建
int check_or_create_wifi_conf() {
    struct stat st;
    if (stat(WIFI_CONF_PATH, &st) == 0) {
        // 文件已存在
        return 0;
    }

    // 创建空配置文件
    FILE *fp = fopen(WIFI_CONF_PATH, "w");
    if (!fp) {
        perror("Failed to create device_wifi.conf");
        return -1;
    }
    fclose(fp);
    return 0;
}

// 写入 SSID 和密码到文件
int write_wifi_conf(const char *ssid, const char *password) {
    FILE *fp = fopen(WIFI_CONF_PATH, "w");
    if (!fp) {
        perror("Failed to open device_wifi.conf for writing");
        return -1;
    }

    fprintf(fp, "SSID=%s\n", ssid);
    fprintf(fp, "PASSWORD=%s\n", password);
    fclose(fp);
    return 0;
}

// 从文件读取 SSID，比较当前连接的 WiFi 名称
int compare_ssid_with_file() {
    char file_ssid[128] = {0};

    FILE *fp = fopen(WIFI_CONF_PATH, "r");
    if (!fp) {
        perror("Failed to open device_wifi.conf");
        return -1;
    }

    char line[256];
    while (fgets(line, sizeof(line), fp)) {
        if (strncmp(line, "SSID=", 5) == 0) {
            strncpy(file_ssid, line + 5, sizeof(file_ssid) - 1);
            file_ssid[strcspn(file_ssid, "\n")] = 0;
            break;
        }
    }
    fclose(fp);

    get_current_wifi();
  

    my_zlog_debug("File SSID: %s", file_ssid);
    my_zlog_debug("Current SSID: %s", current_ssid);

    if (strcmp(file_ssid, current_ssid) == 0) {
        my_zlog_debug("WiFi matches the config file.");
        return 1;
    } else {
        my_zlog_debug("WiFi does NOT match the config file.");
        return 0;
    }
}